# AnCIble - CI for Ansible

## About

This repository provides _highly opinionated_ [Ansible][] templates both for:

* [CI/CD pipelines][pipelines].
* [Roles][] and [playbooks][].

As [Ansible][] CI testing can very complicated, this project was created to
simplify and centralize this procedure.

It relies on [Ansible Lint][], [Ansible Molecule][] and [Ansible Pytest][].

[pipelines]: https://docs.gitlab.com/ee/ci/pipelines/
[Ansible]: https://ansible.com
[Roles]: https://docs.ansible.com/ansible/latest/playbook_guide/playbooks_reuse_roles.html
[playbooks]: https://docs.ansible.com/ansible/latest/playbook_guide/index.html

[Ansible Lint]: https://ansible.readthedocs.io/projects/lint/
[Ansible Molecule]: https://ansible.readthedocs.io/projects/molecule/
[Ansible Pytest]: https://ansible.readthedocs.io/projects/pytest-ansible/

## HOWTOs

This HOWTO assumes an AnCIble setup using a [CI template][], which will
work if you're hosting your [Ansible][] repository at [gitlab.torproject.org][]. Other
installation methods are detailed below.

### CI for existing Ansible roles

First, add the following at your [.gitlab-ci.yml][] file:

```yaml
variables:
  ROLE: your-role-name # customize as needed

include:
  # Customize the project location depending on the GitLab instance in use
  - project: tpo/onion-services/ansible/ancible
    file:
      - .gitlab-ci-base.yml

lint:
  extends: .ancible_lint

molecule_podman:
  extends: .ancible_molecule_podman
```

This will use [AnCIble's base CI template][base-template] to create a `lint` (using [Ansible Lint][])
and a `molecule_podman` (using [Ansible Molecule][]) CI jobs.

[base-template]: https://gitlab.torproject.org/tpo/onion-services/ansible/ancible/-/blob/main/.gitlab-ci-base.yml

Optionally, create a `molecule` folder. You can copy AnCIble's
[molecule](molecule) folder and customize according to your needs, especially
the `roles` configuration on the `converge.yml` files, ensuring Molecule will
process your role:

```yaml
  roles:
    - your-role-name
```

If you don't create this `molecule` folder, AnCIble will use a default.

The next step is to make sure your repository have CI enabled (usually under
"Settings -> General -> Visibility, project features, permissions").

Finally, commit these new files and push to your remote repository.

It's as easy as that! Once these changes are pushed, [GitLab CI][] will kick in
and test your [Ansible][] codebase.

[CI template]: https://gitlab.torproject.org/tpo/web/ancible/-/blob/main/.gitlab-ci-base.yml
[gitlab.torproject.org]: https://gitlab.torproject.org
[.gitlab-ci.yml]: https://docs.gitlab.com/ee/ci/yaml/
[GitLab CI]: https://docs.gitlab.com/ee/ci/

### CI for existing Ansible playbooks

For existing playbook repositories, the procedure is similar with the above,
with the following differences.

The [.gitlab-ci.yml][] does not need to define the `ROLE` variable, and Pytest
is used instead of Molecule:

```yaml
include:
  - project: tpo/onion-services/ansible/ancible
    file:
      - .gitlab-ci-base.yml

playbook_lint:
  extends: .ancible_playbook_lint

pytest_podman:
  extends: .ancible_pytest_podman
```

Also, you won't need to create a `molecule` folder.

**CAVEAT** currently, CI for existing playbooks is limited to repository layouts
where the roles are stored inside the `playbooks/roles` folder. This limitation is
tracked at [tpo/onion-services/ansible/ancible#3][].

[tpo/onion-services/ansible/ancible#3]: https://gitlab.torproject.org/tpo/onion-services/ansible/ancible/-/issues/3

## Examples

There's an [example role][] and an [example playbook][] with the most basic
configuration needed to have working CI through AnCIble.

AnCIble is activelly used by the following projects (as of 2025-02-13), which
can be used as examples as well:

* [Onionprobe Ansible Role][]
* [Onionspray Ansible Role][]
* [Basic Ansible Role][]
* [Docker Ansible Role][]

The [base template][base-template] is also a good reference in what is possible
with AnCIble.

[example role]: https://gitlab.torproject.org/tpo/onion-services/ansible/example-role
[example playbook]: https://gitlab.torproject.org/tpo/onion-services/ansible/example-playbook
[Basic Ansible Role]: https://gitlab.torproject.org/tpo/onion-services/ansible/basic-role
[Docker Ansible Role]: https://gitlab.torproject.org/tpo/onion-services/ansible/docker-role
[Onionprobe Ansible Role]: https://gitlab.torproject.org/tpo/onion-services/ansible/onionprobe-role
[Onionspray Ansible Role]: https://gitlab.torproject.org/tpo/onion-services/ansible/onionspray-role

## Repository setup

This repository is ready to be used in existing repositories
It can be installed in a number of ways as follows.

### Forking the project

Simply fork this project and change whatever you need.

It's ready to be used as an [Ansible][] role, but with a little tweaking can be used
for playbooks and collections as well.

### Manually copying

Even simpler, copy all the relevant files from this repository into your project.

### Vendorizing

You can also vendorize AnCIble by reserving a folder for it like in a
`vendor` (or `vendors`) inside your project.

#### Vendorization strategies

There are several ways to vendor it:

0. Use AnCIble directly from it's remote repository (with GitLab CI
   templating functionality).

1. As a Git submodule: simply put it as a Git submodule somewhere:

        mkdir -p vendors && \
        git submodule add --depth 1 https://gitlab.torproject.org/tpo/onion-services/ansible/ancible vendors/ancible

2. Regular cloning:

        mkdir -p vendors && \
        git clone --depth 1 https://gitlab.torproject.org/tpo/onion-services/ansible/ancible vendors/ancible && \
        git ignore vendors/ancible

3. Using `git-subtree(1)`.

#### Configuration

After vendorization, make sure to create the needed configuration files.

You can either copy some of these files from the AnCIble repository or use
symbolic links to reference all the needed files from the root of your
repository.

The recommended files can be set as follows:

```shell
cp -a vendors/ancible/molecule .
sed -i -e "s/ancible/YOUR-ROLE-NAME/g" molecule/local/converge.yml
sed -i -e "s/ancible/YOUR-ROLE-NAME/g" molecule/podman/converge.yml
```

##### CI

<!-- Use this once GitHub Actions is available -->
<!--
If you plan to use either GitLab CI/CD or GitHub Actions to build your
documentation, make sure to at least copy/adapt the corresponding files from
the submodule. You can't just symlink those since the submodule won't be
accessible when GitLab and GitHub looks for CI/CD or Actions files. The
provided GitLab CI/CD and GitHub actions configuration takes care of
initializing the submodule during build time, but they should be available in
the main repo in order to do so.
-->
If you plan to use either GitLab CI/CD to build your documentation, make sure
to at least copy/adapt the corresponding files from the submodule. You can't
just symlink those since the submodule won't be accessible when GitLab looks
for CI/CD files. The provided GitLab CI/CD configuration takes care of
initializing the submodule during build time, but they should be available in
the main repo in order to do so.

The following `gitlab-ci.yml` configuration example uses a CI template and does
not require you to manually vendorize AnCIble in a subfolder (but requires that
AnCIble is reachable at `tpo/onion-services/ansible/ancible` in the GitLab
instance):

```yaml
variables:
  ROLE: your-role-name

include:
  # Customize the project location depending on the GitLab instance in use
  - project: tpo/onion-services/ansible/ancible
    file:
      - .gitlab-ci-base.yml

lint:
  extends: .ancible_lint

molecule_podman:
  extends: .ancible_molecule_podman
```

## How to pronounce?

How AnCIble is pronounced? I don't know!
Maybe An-CI-ble (_an-see-eye-bow_) is best to distinguish from Ansible (_an-see-bow_).
